#if !defined(_MGR_INFO_H_)
#define _MGR_INFO_H_
/*
 * Licensed Materials - Property of IBM
 * Rational ClearCase
 * (C) Copyright IBM Corp. 1991, 2014.  All Rights Reserved.
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp
 */

/*---------------------------------------------------------------------------
 * ClearCase Type Manager Interface
 * 
 * A type manager is a suite of programs used to manipulate element version
 * data with a particular format.  Many ClearCase actions, such as checkin,
 * involve updating or accessing a version's data in the VOB storage directory
 * as well as its meta-data in the VOB database.  Whenever such an action
 * requires an understanding of the data format, ClearCase software invokes
 * the appropriate type manager operation.  Each element has an element type.
 * The element type identifies by name the type manager to use to process data
 * for that element's versions.  The type may specify the type manager directly
 * or inherit it from a supertype.
 *
 * To add a new type manager to the system, in each ClearCase release area
 * tree, add directory:
 *   <atria-home-dir>/lib/mgrs/<manager-name>
 * and add a program for each operation supported by the manager:
 *   <atria-home-dir>/lib/mgrs/<manager-name>/<operation-name>
 * where:
 *   <atria-home-dir> is the release directory (e.g. /opt/rational/clearcase)
 *   <manager-name> is the name of the manager given in the element type
 *   <operation-name> is the name of the operation (see MGR_OP_* below)
 *
 * An operation can be implemented as a program, shell script, or symbolic
 * link to another program (in the same manager, another manager, or anywhere 
 * else).  For example, the built-in text_file_delta manager links its compare
 * operation to the cleardiff program.  Several operations may link to a
 * common program that checks the leaf of argv[0] or $0 to see what specific
 * operation is requested.  ClearCase invokes a type manager at the operation
 * pathname with ASCII string arguments (described below) which depend on the
 * operation.  The operation returns an exit status indicating various kinds
 * of success or failure.  If the manager does not support an operation, omit
 * that operation from the manager's directory.
 *
 * Compare and merge operations manipulate cleartext data for versions.
 * However, most operations manipulate source containers.  Only a type manager
 * knows the format of the source containers for the versions it manages.  A
 * manager may store one version or many versions in each source container.
 * In most cases, ClearCase passes the manager an old (or predecessor)
 * container and a new container.  The old container holds the data for one
 * or more versions as they currently exist.  At the time of the operation, the
 * VOB database still refers to that container for those existing versions.
 * The manager must not modify the old container, since that container must
 * survive until the update finishes or if the update does not finish.  The
 * type manager performs the requested data update in the new container, which
 * ClearCase has already created with appropriate ownership and permissions.
 * If the manager completes the update successfully, ClearCase updates the VOB
 * as indicated by the manager's return value.  If the manager fails, ClearCase
 * cleans up and fails the higher-level action.
 * 
 * For example, when a user checks in a new version of type "text_file",
 * ClearCase checks the VOB to make sure the checkin is valid, gathers
 * information to pass to the type manager, creates a new container, and
 * invokes the text_file_delta type manager to store the new version.  The
 * text_file_delta manager saves all the data from the predecessor version's
 * container in the new container and incorporates the data for the new
 * version, then returns a status indicating the the new container should be
 * used and the old should be discarded.  ClearCase removes the write rights
 * from the new container, updates the VOB database to record the checkin and
 * to change all versions that used the old container to use the new container,
 * then deletes the old container.  If the type manager or the VOB update
 * fails, ClearCase instead deletes the new container, leaves the database
 * unchanged, and fails the checkin.
 * 
 * Also see <atria-home-dir>/lib/mgrs/mgr_info.sh for definitions for shell
 * scripts.
 */


/*---------------------------------------------------------------------------
 * Operations
 * Manager operation names.  See command line interface below for more
 * information on what the operations do.
 *
 * MGR_OP_ANNOTATE	Operation called when someone fetches annotated data
 *			for a version (e.g. "cleartool annotate").

 * MGR_OP_COMPARE	Operation called to compare two or more versions
 *			(e.g. "cleartool diff").
 *
 * MGR_OP_CONSTRUCT_VERSION
 *			Operation called when someone tries to read a version's
 *			data (cleartext construction).  The manager fetches
 *			the requested version's data and stores it in a
 *			supplied cleartext container.
 *
 * MGR_OP_CONTEXT_MERGE
 *			Operation called after deliver or rebase merge a
 *			set of files manageed by this type manager, for
 *			post-processing of the files as a group.
 *
 * MGR_OP_CREATE_BRANCH	Operation called when a new branch is created (e.g.
 *			"cleartool mkbranch").  The manager creates or returns
 *			data for version 0 on the new branch, which is
 *			identical to the data of the version from which the
 *			branch sprouts.
 *
 * MGR_OP_CREATE_ELEMENT
 *			Operation called when a new element is created (e.g.
 *			"cleartool mkelem").  The manager creates a data
 *			container to represent the empty initial version
 *			(/main/0).
 * 
 * MGR_OP_CREATE_VERSION
 *			Operation called when a new version <2GB is created
 *			(e.g. "cleartool checkin").  The manager stores the
 *			data for the new version.
 *
 * MGR_OP_CREATE_VERSION64
 *			Operation called when a new version >=2GB is created
 *			(e.g. "cleartool checkin").  The manager stores the
 *			data for the new version.
 *
 * MGR_OP_DELETE_BRANCHES_VERSIONS
 *			Operation called when one or more branches and/or
 *			versions are deleted (e.g. "cleartool rmbranch",
 *			"rmver").  The manager deletes the data for the
 *			requested branches and versions.
 *
 * MGR_OP_GET_CONT_INFO Operation called to inquire information about
 *                      a container. The manager returns info about the
 *                      versions, etc in the container.
 *
 * MGR_OP_MERGE		Operation called to merge two or more versions
 *			(e.g. "cleartool merge").
 *
 * MGR_OP_XCOMPARE	Operation called to graphically compare two or more
 *			versions (e.g. "cleartool xdiff").
 *
 * MGR_OP_XMERGE	Operation called to graphically merge two or more
 *			versions (e.g. "cleartool xmerge").
 */

#define MGR_OP_CREATE_ELEMENT	 "create_element"
#define MGR_OP_CREATE_BRANCH	 "create_branch"
#define MGR_OP_CREATE_VERSION	 "create_version"
#define MGR_OP_CREATE_VERSION64  "create_version64"
#define MGR_OP_CONSTRUCT_VERSION "construct_version"
#define MGR_OP_DELETE_BRANCHES_VERSIONS \
				 "delete_branches_versions"
#define MGR_OP_GET_CONT_INFO	 "get_cont_info"
#define MGR_OP_COMPARE		 "compare"
#define MGR_OP_XCOMPARE		 "xcompare"
#define MGR_OP_MERGE		 "merge"
#define MGR_OP_XMERGE		 "xmerge"
#define MGR_OP_CONTEXT_MERGE	 "context_merge"
#define MGR_OP_ANNOTATE		 "annotate"


/*---------------------------------------------------------------------------
 * Command line options.
 * 
 * MGR_FLAG_*		Command line option keywords passed to type manager
 *			operations.
 */

#define MGR_FLAG_ABORT		 "-abort"
#define MGR_FLAG_ALL		 "-all"
#define MGR_FLAG_BASE		 "-base"
#define MGR_FLAG_BRANCH_OIDS	 "-branches"
#define MGR_FLAG_COLUMNS    	 "-columns"
#define MGR_FLAG_DIFF	    	 "-diff"
#define MGR_FLAG_DIRECTORY	 "-directory"
#define MGR_FLAG_FNAME           "-fname"
#define MGR_FLAG_HORIZONTAL	 "-hstack"
#define MGR_FLAG_IGNORE_BLANKS	 "-blank_ignore"
#define MGR_FLAG_OUT		 "-out"
#define MGR_FLAG_PARALLEL	 "-parallel"
#define MGR_FLAG_QALL		 "-qall"
#define MGR_FLAG_Q		 "-q"
#define MGR_FLAG_SERIAL	    	 "-serial"
#define MGR_FLAG_TINY	    	 "-tiny"
#define MGR_FLAG_VER_OIDS	 "-versions"
#define MGR_FLAG_VERTICAL	 "-vstack"
#define MGR_FLAG_WINDOW	    	 "-window"
#define MGR_FLAG_QNTRIVIAL        "-qntrivial"


/*---------------------------------------------------------------------------
 * Return values
 * Each manager returns a value indicating its result.  If a program or shell
 * script exits with a value "n", it is returned to the ClearCase invoking
 * program shifted 8 bits to the left.  The type manager mechanism compensates
 * for this, so managers should simply return the values below with "exit (n)"
 * or "exit n" (/bin/sh).
 *
 * Return values have been selected so that managers can easily be shell
 * scripts.  Operations like diff/merge use 0 or 1 to indicate identical/ok
 * vs. diffs.  Complex operations return larger values so shell scripts can
 * use "#!/bin/sh -e" and exit on error with a Unix status (<32) when there is
 * problem.  These small status codes, as well as any other unexpected return
 * value, will be treated as a failure.
 *
 * mgr_rtn_value_t	Value returned by type manager.
 *
 * MGR_COMPARE_DIFF_OR_ERROR
 *			Return value for compare operation, indicating that
 *			the comparison was not successful or that the
 *			comparison was successful but there were differences.
 * MGR_COMPARE_NODIFFS	Return value for compare operation, indicating
 *			that the comparison was successful and there were no
 *			differences.
 *
 * MGR_CONSTRUCT_USE_NEW_FILE
 *			Return value for construct operation, indicating that
 *			the cleartext has been constructed at the supplied
 *			pathname.
 * MGR_CONSTRUCT_USE_SRC_CONTAINER
 *			Return value for construct operation, indicating that
 *			the source container can be used as the cleartext for
 *			the version.
 *
 * MGR_DELETE_KEEP_JUST_NEW
 *			Return value for delete operation, indicating that
 *			the old container should be discarded and the new
 *			container should be used.
 * MGR_DELETE_KEEP_NEITHER
 *			Return value for delete operation, indicating that
 *			both the old container and the new container should
 *			be discarded.
 * MGR_DELETE_KEEP_JUST_OLD
 *			Return value for delete operation, indicating that
 *			the old container should be used and the new container
 *			is not needed.
 *
 * MGR_FAILED		Value indicating operation failed.  Any unexpected
 *			return value is also treated as a failure.
 *
 * MGR_FAILED_NO_PRED_CONT
 *                      Return value for create operation, indicating failure
 *                      as previous container does not exist.  Common case if
 *                      container was renamed before manager could open it.
 * MGR_INFO_NO_INFO     Return value for 'get cont info' operation, indicating
 *			that the manager doesn't maintain version identification
 *                      information. Such a manager's containers can only
 *                      hold one version data image (though multiple versions
 *                      that have identical data -e.g., version 0 of a branch
 *                      and the branch's branch point- can share the container.
 *
 * MGR_MERGE_ABORT	Return value for merge/xmerge operation, indicating
 *			that the merger was aborted due to conflicts.
 * MGR_MERGE_OK		Return value for merge/xmerge operation, indicating
 *			that the merger was completed successfully.
 *
 * MGR_OK		Value indicating operation succeeded.
 *
 * MGR_STORE_FILES_ARE_IDENT
 *			Special value to OR into a MGR_STORE_* status to
 *			indicate that the version stored has the same data as
 *			its predecessor.
 * MGR_STORE_KEEP_BOTH	Return value for create operation, indicating that
 *			both the old and new containers should be kept.
 *			A new container has been created to store the requested
 *			update.  Any versions that used the old container
 *			should continue to use the old container.  The new
 *			version should use the new container.
 * MGR_STORE_KEEP_JUST_NEW
 *			Return value for create operation, indicating that
 *			the old container should be discarded and the new
 *			container should be used.  The new container has all
 *			the information in the old container plus the requested
 *			update.  All versions that used the old container
 *			should now use the new container.  The old container
 *			can then be discarded.
 * MGR_STORE_KEEP_JUST_OLD
 *			Return value for create operation, indicating that
 *			the old container should be saved, and the new
 *			container is not needed.  Any versions that used the
 *			old container should continue to use the old container.
 * MGR_STORE_KEEP_NEITHER
 *			Return value for create operation, indicating that
 *			the old container, if any, can be discarded and the new
 *			container is not needed.
 */

typedef int mgr_rtn_value_t;

#define MGR_OK				0
#define MGR_FAILED			1
#define MGR_FAILED_NO_PRED_CONT         2

#define MGR_STORE_KEEP_NEITHER		101
#define MGR_STORE_KEEP_JUST_OLD		102
#define MGR_STORE_KEEP_JUST_NEW		103
#define MGR_STORE_KEEP_BOTH		104
#define MGR_STORE_FILES_ARE_IDENT	(1<<7)

#define MGR_CONSTRUCT_USE_SRC_CONTAINER	101
#define MGR_CONSTRUCT_USE_NEW_FILE	102

#define MGR_DELETE_KEEP_NEITHER		101
#define MGR_DELETE_KEEP_JUST_OLD	102
#define MGR_DELETE_KEEP_JUST_NEW	103

#define MGR_INFO_NO_INFO                101

#define MGR_COMPARE_NODIFFS		MGR_OK
#define MGR_COMPARE_DIFF_OR_ERROR	MGR_FAILED

#define MGR_MERGE_OK			MGR_OK
#define MGR_MERGE_ABORT			MGR_FAILED

/*---------------------------------------------------------------------------
 * Macros to assist in processing manager command line arguments.
 */

#define MGR_ARG_GET_STR(opname, s, argv, ap) { \
	s = argv[ap++]; \
	if (s == NULL) { \
	    imsg_print (IMSG_ERR, TBS_CMSG_MGR_BAD_SYNTAX, opname); \
	    imsg_print (IMSG_ERR, TBS_CMSG_MGR_USAGE); \
	    return MGR_FAILED; \
	} \
}

#define MGR_ARG_GET_OIDSTR(opname, s, argv, ap) \
        MGR_ARG_GET_STR(opname, s, argv, ap)

#define MGR_ARG_GET_INT(opname, valp, argv, ap) { \
	char *tmp_valstr; \
	MGR_ARG_GET_STR(opname, tmp_valstr, argv, ap); \
	valp = strtol(tmp_valstr, NULL, 10); \
}

#define MGR_ARG_GET_HEXINT(opname, valp, argv, ap) { \
	char *tmp_valstr; \
	MGR_ARG_GET_STR(opname, tmp_valstr, argv, ap); \
	valp = strtol(tmp_valstr, NULL, 16); \
}

/*---------------------------------------------------------------------------
 * Annotation
 * 
 * MGR_ANNOTATE_*	In output file of "annotate" operation, this code at
 *			the beginning of the line indicates the relationship
 *			of a line of data to a particular version.
 *  MGR_ANNOTATE_DELETED
 *			This line of data was included in some predecessor of
 *			the version, but has been deleted.
 *			Line format: D added-ver-oid,deleted-ver-oid text
 *  MGR_ANNOTATE_INCLUDED
 *			This line of data is included in the version.
 *			Line format: I added-ver-oid text
 *  MGR_ANNOTATE_UNRELATED
 *			This line of data is unrelated to the version, that is,
 *			it was introduced in some version that is not a
 *			predecessor of the version (i.e., a descendant or an
 *			unrelated branch).
 *			Line format: I added-ver-oid text
 */

typedef char * mgr_annotate_code_t;
#define MGR_ANNOTATE_INCLUDED	"I"
#define MGR_ANNOTATE_DELETED	"D"
#define MGR_ANNOTATE_UNRELATED	"U"


/*---------------------------------------------------------------------------
 * Directory Diff/Merge
 *
 * This is the file format used for comparison and merging of VOB directories.
 * 
 *   MGR_DDIFF_FMT_STR MGR_DDIFF_FORMAT
 *   directory-pathname
 *   name oid ftype slink-text create-time create-user
 *   ...
 *   
 * directory-pathname	pathname of directory
 * name			name of element or symbolic link in this directory
 * oid			object id of element or symbolic link
 * ftype		type of object (mgr_ddiff_ftype_t)
 * slink-text		link text if ftype is symbolic link (MGR_DDIFF_SLINK),
 *			or "0:" if none
 * create-time		time in seconds since 1 January 1970 when object was
 *			created, or "0:" if not available
 * create-user		user name of user who created object, or "0:" if not
 *			available
 *
 * All items are encoded as "length:text" (e.g., "Hello" is "5:Hello"; 
 * "23" is "2:23").
 * 
 * Example file:
 *   DDIFF_Format: 1
 *   14:mydir@@/main/0
 *   7:hello.c 40:f14cfb45.69bf11cc.b588.00:01:53:00:e8:c3 1:1 0: 8:76567550 4:bill
 *   8:test.dir 40:91b00003.b26411ca.adf1.08:00:69:02:1d:c7 1:2 0: 8:76563423 4:jane
 *   5:msg.h 40:3f57df5d.4c0d11cd.a00f.08:00:69:02:1d:c7 1:3 16:../include/msg.h 8:76567664 4:bill
 *   4:hi.c 40:f14cfb45.69bf11cc.b588.00:01:53:00:e8:c3 1:1 0: 0: 0:
 *
 * MGR_DDIFF_FMT_STR	Keyword identifying file format.
 * MGR_DDIFF_FORMAT	File format version number.
 *
 * mgr_ddiff_ftype_t	Type of object.
 *  MGR_DDIFF_DIR	Directory element.
 *  MGR_DDIFF_FILE	File element.
 *  MGR_DDIFF_SLINK	Symbolic link.
 */

typedef enum {
    MGR_DDIFF_FILE = 0,
    MGR_DDIFF_DIR = 1,
    MGR_DDIFF_SLINK = 2
} mgr_ddiff_ftype_t;

#define MGR_DDIFF_FMT_STR "DDIFF_Format:"
#define MGR_DDIFF_FORMAT 1

/*---------------------------------------------------------------------------
 * Container Description Information
 *
 * MGR_INFO_*	        Definitions of the record types for the 
 *                      'get_cont_info' operation.
 *
 * Refer to the description of the create_{element,branch,version} 
 * operations for a definition of the representation of the various 
 * field types / values used in the information records.
 * The information record type string must be the first character
 * on a line.
 * 
 * If a manager is unable to provide actual values for each of the
 * optional fields, it must provide the NIL field value.
 *
 *  MGR_INFO_VERSION
 *			A version information record.
 *			Line format: 
 *                        V ver_oid [ver_num] [branch_oid] [create_time]
 *  MGR_INFO_BRANCH
 *			A Branch information record.
 *			Line format: 
 *                        B branch_oid [pred_branch_oid] [pred_ver_oid] \
 *                          [pred_ver_num] [create_time]
 *
 *                      Note: the optional pred_ fields for the 
 *                      initial branch's branch information record 
 *                      (i.e., /main) should have the nil field value.
 *  MGR_INFO_ELEMENT
 *			An Element information record.
 *			Line format: 
 *                        E elem_oid [initial_branch_oid] [create_time]
 *  MGR_INFO_NIL_FIELD  
 *                      The 'nil' string value ('no info provided') for an 
 *                      optional field.
 */

typedef char * mgr_info_code_t;
#define MGR_INFO_VERSION        "V"
#define MGR_INFO_BRANCH         "B"
#define MGR_INFO_ELEMENT        "E"
#define MGR_INFO_NIL_FIELD      "NIL"




/*---------------------------------------------------------------------------
 * Command Line Interface
 * This defines the command line interface for each type manager operation.
 * All arguments are ASCII strings.
 *
 * General explanation of arguments:
 *
 * create_time		The creation time of an object is given as extra
 *			information.  If a manager stores multiple versions in
 *			a container, this information can be used by a utility
 *			to locate data for versions created since a given date.
 *			Such a utility can help recover data for versions when
 *			the VOB database is lost or is older than the source
 *			pools (e.g. due to a disk crash or data loss after
 *			failed backups).
 *
 * new_container_pname	The pathname of a new container designates an existing
 *			object in which the manager should store the requested
 *			updated data, if needed.
 *
 * oid			An object identifier (OID) is a unique name for an
 *			element, branch, or version.  The manager may not need
 *			the OID unless it stores multiple versions in the
 *			same source container.  OIDs described below as
 *			"future" identify objects that do not yet exist in the
 *			VOB database.
 *
 * old_container_pname	
 * pred_container_pname	The pathname of an existing source container is used
 *			by existing versions and must not by modified by the
 *			type manager.  These containers may not be needed if
 *			the manager does not store multiple versions in a
 *			container.
 *
 * ver_num		The version number of a version is given as extra
 *			information to aid debugging and recovery of data.
 */

/****************************************************************************
 * create_element
 * Record the creation of a new element, its main branch and initial version.
 * Store data for an empty initial version of the element (e.g. /main/0).
 *
 * Command line:
 *  create_element create_time element_oid initial_branch_oid zero_ver_oid
 *                 new_container_pname
 * 
 * Arguments:
 *  create_time		Date/time when element created, in seconds since 
 *			January 1, 1970, formatted as hex string.
 *  element_oid		Object id of future element.
 *  initial_branch_oid	Object id of future initial branch of element.
 *  zero_ver_oid	Object id of future version 0 on initial branch.
 *  new_container_pname	Pathname of container in which to store initial empty
 *			version.
 *
 * Result:
 *  MGR_STORE_KEEP_JUST_NEW
 *			Version was successfully stored in new container.
 *  MGR_FAILED		Manager operation failed.
 */

/****************************************************************************
 * create_branch
 * Record the creation of a new branch and version zero on that branch.
 * The data for version zero is identical to the data for the predecessor
 * version.  Add the new branch and version zero to source container, or reuse
 * the predecessor's container.
 *
 * See the construct_version operation description for requirements
 * regarding exact preservation of version data.
 *
 * Command line:
 *  create_branch create_time new_branch_oid zero_ver_oid new_container_pname
 *		  pred_branch_oid pred_ver_oid pred_ver_num
 *		  pred_container_pname
 * 
 * Arguments:
 *  create_time		Date/time when branch created, in seconds since 
 *			January 1, 1970, formatted as hex string.
 *  new_branch_oid	Object id of future branch.
 *  zero_ver_oid	Object id of future version 0 on new branch.
 *  new_container_pname	Pathname of container in which to add new branch and
 *			version 0.
 *  pred_branch_oid	Object id of branch of predecessor version.
 *  pred_ver_oid	Object id of predecessor version from which new branch
 *			sprouts.
 *  pred_ver_num	Version number of predecessor version, formatted as
 *			decimal string.
 *  pred_container_pname
 *			Pathname of predecessor version's storage container.
 *
 * Result:
 *  MGR_STORE_KEEP_BOTH	Both the old and new containers should be kept.  The
 *			new version should use the new container.  Old versions
 *			should continue to use the old container.
 *  MGR_STORE_KEEP_JUST_NEW
 *			The old container should be discarded and the new
 *			container should be used for this version and any
 *			other versions that used the old container.
 *  MGR_STORE_KEEP_JUST_OLD
 *			The old container should be saved but the new
 *			container is not needed.  The new version should use
 *			the old container.
 *  MGR_FAILED		Manager operation failed.
 */

/****************************************************************************
 * create_version
 * Store the data for a new version.
 * Store the version's data in in the supplied new container, combining it
 * with the predecessor's data if desired (e.g for incremental deltas).
 *  
 * Warning: In some situations create_version is called with a branch_oid
 *	    of a branch that the manager has not yet been informed of.  In 
 *          this case the manager should be prepared to create that branch
 *          as needed.  Furthermore, when this occurs it is legal for
 *          the new version to be version 0 on the branch (new_ver_num == 0).
 *
 * See the construct_version operation description for requirements
 * regarding exact preservation of version data.
 *
 * Command line:
 *  create_version create_time branch_oid new_ver_oid new_ver_num
 *		   new_container_pname pred_branch_oid pred_ver_oid
 *		   pred_ver_num pred_container_pname data_pname
 *
 * Arguments:
 *  create_time		Date/time when version created, in seconds since 
 *			January 1, 1970, formatted as hex string.
 *  branch_oid		Object id of branch where new version will be added.
 *  new_ver_oid		Object id of future version.
 *  new_ver_num		Version number of new version, formatted as decimal
 *			string.
 *  new_container_pname	Pathname of container in which to store new version's
 *			data.
 *  pred_branch_oid	Object id of branch of predecessor version.
 *  pred_ver_oid	Object id of predecessor version.
 *  pred_ver_num	Version number of predecessor version, formatted as
 *			decimal string.
 *  pred_container_pname
 *			Pathname of predecessor version's storage container.
 *  data_pname		Pathname of data for new version.
 *
 * Result:
 *  MGR_STORE_KEEP_BOTH	Both the old and new containers should be kept.  The
 *			new version should use the new container.  Old versions
 *			should continue to use the old container.
 *  MGR_STORE_KEEP_JUST_NEW
 *			The old container should be discarded and the new
 *			container should be used for this version and any
 *			other versions that used the old container.
 *  MGR_STORE_FILES_ARE_IDENT
 *			Value to OR into above result if data for version
 *			stored is identical to data of predecessor version
 *			(optional).
 *  MGR_FAILED		Manager operation failed.
 */

/****************************************************************************
 * construct_version
 * Fetch the data for a version.
 * Extract the data for the requested version into the supplied pathname, or
 * return a value indicating that the source container can be used as the
 * cleartext data for the version.
 *
 * NOTE: The type manager must preserve the version data 
 * provided to it on the create_version operation (and the implicit 
 * version data for a create_branch operation) such that the 
 * construct_version operation yields _exactly_ the same version
 * data that was originally provided.
 * 
 * Command line:
 *  construct_version source_container_pname data_pname version_oid
 *
 * Arguments:
 *  source_container_pname
 *			Pathname of source container for the version.
 *  data_pname		Pathname in which to store extracted data for version.
 *  version_oid		Object id of version to fetch.
 *
 * Result:
 *  MGR_CONSTRUCT_USE_SRC_CONTAINER
 *			Use source container as data instead of data_pname.
 *  MGR_CONSTRUCT_USE_NEW_FILE
 *			Data was successfully fetched to data_pname.
 *  MGR_FAILED		Manager operation failed.
 */

/****************************************************************************
 * delete_branches_versions
 * Delete one or more branches and/or versions all from the same container.
 * Delete each branch and its corresponding version zero, and delete each
 * version.  Store the resulting data in the supplied source container, or if
 * no data remains after the deletions, return a result indicating that the
 * old container can be discarded and the new container is not needed.
 *
 * Managers are REQUIRED to actually delete versions that have no
 * living descendents (i.e., to prune the version tree).  This is to
 * ensure that the manager will behave correctly following certain recovery
 * actions. Specifically, under those conditions, the manager can
 * later be presented with a create_version operation for a version number 
 * that the manager had previously been told to create and delete.
 * The version's oid and data may be different than in the original instance.
 *
 * This apparent 'reuse' of the version number can only occur for 
 * deleted versions with no living descendents.  Managers are allowed
 * to merely tag delete versions that don't meet the 'no living 
 * descendents' criteria. Such managers are REQUIRED to meet the
 * initial requirement following *any* delete_branches_versions
 * operation.  That is, previously tag-deleted versions that no longer
 * have any living descendents must be deleted (regardless of whether
 * or not the current delete_branches_versions operation affects any
 * of those versions).
 *
 * The manager must be prepared to be asked to delete branch and/or version
 * oids that it may not know about.  It must silently accept these
 * (the manager will meet the requirement that the branch / version
 * does not exist in the container).
 * 
 * Command line:
 *  delete_branches_versions old_container_pname new_container_pname
 *			     [-branches branch_oid ...]
 *			     [-versions ver_oid ...]
 *
 * Arguments:
 *  old_container_pname	Pathname of source container that currently contains
 *			the data for the branches/versions.
 *  new_container_pname	Pathname of new source container in which to store
 *			data without the deleted branches/versions.
 *  -branches branch_oid ...
 *			List of object ids of branches to delete.
 *  -versions ver_oid ...
 *			List of object ids of versions (includeing zero) to
 *			delete. 
 *
 * Result:
 *  MGR_DELETE_KEEP_JUST_NEW
 *			The old container should be discarded and the new
 *			container should be used for all remaining versions
 *			that used the old container before the deletion.
 *  MGR_DELETE_KEEP_NEITHER
 *			The old container can be discarded and the new
 *			container is not needed.
 *  MGR_DELETE_KEEP_JUST_OLD
 *			The old container should be used for all remaining
 *			versions, and the new container is not needed.
 *			(This result is not recommended.  It should be used
 *			only for managers who cannot delete the data from the
 *			container.  The branches/versions will be deleted from
 *			the database without ever reclaiming the space in the
 *			source container.)
 *  MGR_FAILED		Manager operation failed.
 */

/****************************************************************************
 * compare
 * Compare the data for two or more versions.
 * For more information, see cleartool diff.
 *
 * Command line:
 *  compare [-tiny | -window] [-serial | -diff | -parallel] [-columns n]
 *	    [-directory] [-blank_ignore] [pass-through-options]
 *          [-fname string ...] pname pname ...
 *
 * Arguments:
 *  [-tiny | -window] [-columns n]
 *			See corresponding cleartool diff options.
 *  -serial		See cleartool diff -serial_format.
 *  -diff		See cleartool diff -diff_format.
 *  -parallel		Requests side-by-side display of differences (e.g.
 *			cleartool diff default).
 *  -directory		Indicates that contributors are VOB directory versions
 *			represented by encoded files in format described under
 *			"Directory Diff/Merge" above.
 *  -blank_ignore       See cleartool diff -blank_ignore.
 *  pass-through-options
 *			Options specific to this type manager.
 *  -fname string ...   String to display for an input pathname. This option
 *                      can be repeated up to the number of input pathnames
 *                      supplied. The compare program should use string[n]
 *                      as a label for pname[n], if string[n] was supplied.
 *  pname pname ...	List of pathnames of cleartext data of versions to
 *			compare.
 *
 * Result:
 *  MGR_COMPARE_NODIFFS	The comparison was successful and there were no
 *			differences.
 *  MGR_COMPARE_DIFF_OR_ERROR
 *			The comparison was not successful, or the comparison
 *			was successful but there were differences.
 */

/****************************************************************************
 * xcompare
 * Compare the data for two or more versions, using a graphical display.
 * For more information, see cleartool xdiff.
 *
 * Command line:
 *  xcompare [-hstack | -vstack] [-tiny] [-directory] [-blank_ignore]
 *           [pass-through-options] [-fname string ...] pname pname ...
 *
 * Arguments:
 *  [-hstack | -vstack] [-tiny]
 *			See corresponding xcleartool xdiff option.
 *  -directory		Indicates that contributors are VOB directory versions
 *			represented by encoded files in format described under
 *			"Directory Diff/Merge" above.
 *  -blank_ignore       See cleartool diff -blank_ignore.
 *  pass-through-options
 *			Options specific to this type manager.
 *  -fname string ...   String to display for an input pathname. This option
 *                      can be repeated up to the number of input pathnames
 *                      supplied. The xcompare program should use string[n]
 *                      as a label for pname[n], if string[n] was supplied.
 *  pname pname ...	List of pathnames of cleartext data of versions to
 *			compare.
 *
 * Result:
 *  MGR_COMPARE_NODIFFS	The comparison was successful and there were no
 *			differences.
 *  MGR_COMPARE_DIFF_OR_ERROR
 *			The comparison was not successful, or the comparison
 *			was successful but there were differences.
 */

/****************************************************************************
 * merge
 * Merge the data for two or more versions.
 * For more information, see cleartool merge.
 * When selective query is used (-query switch) it is assumed that the last
 * pname listed is the to-version, unless -to is used, in which case the 
 * pname given with -to is the to-version.
 *
 * Command line:
 *  merge [-qall | -abort] [-base base_pname] -out output_pname
 *	  [-tiny | -window] [-serial | -diff | -parallel] [-columns n]
 *	  [-directory] [pass-through-options]
 *        [-fname string ...] pname pname ...
 *
 * Arguments:
 *  [-qall | -abort] [-tiny | -window] [-columns n]
 *			See corresponding cleartool merge options.
 *  -serial		See cleartool merge -serial_format.
 *  -diff		See cleartool merge -diff_format.
 *  -parallel		Requests side-by-side disply of differences (e.g.
 *			cleartool merge default).
 *  -base base_pname	Pathname of cleartext data for base version, if any.
 *  -out output_pname	Pathname where cleartext data for result of merge
 *			should be stored.
 *  -directory		Indicates that contributors are VOB directory versions
 *			represented by encoded files in format described under
 *			"Directory Diff/Merge" above.
 *  pass-through-options
 *			Options specific to this type manager.
 *  -fname string ...   String to display for an input pathname. This option
 *                      can be repeated up to the number of input pathnames
 *                      supplied, including the base_pname (which is
 *                      considered to be the first pathname.) The merge
 *                      program should use string[n] as a label for pname[n],
 *                      if string[n] was supplied.
 *  pname pname ...	List of pathnames of cleartext data of versions
 *			contributing to merge.
 *
 * Result:
 *  MGR_MERGE_OK	The merger was completed successfully.
 *  MGR_MERGE_ABORT	The merger was aborted due to conflicts.
 *  MGR_FAILED		Manager operation failed.
 */

/****************************************************************************
 * xmerge
 * Merge the data for two or more versions, using a graphical display.
 * For more information, see cleartool xmerge.
 *
 * Command line:
 *  xmerge [-qall | -abort] [-base base_pname] -out output_pname
 *	   [-hstack | -vstack] [-tiny] [-directory] [pass-through-options]
 *         [-fname string ...] pname pname ...
 *
 * Arguments:
 *  [-qall | -abort] [-hstack | -vstack] [-tiny]
 *			See corresponding cleartool xmerge options.
 *  -base base_pname	Pathname of cleartext data for base version, if any.
 *  -out output_pname	Pathname where cleartext data for result of
 *			merge should be stored.
 *  -directory		Indicates that contributors are VOB directory versions
 *			represented by encoded files in format described under
 *			"Directory Diff/Merge" above.
 *  pass-through-options
 *			Options specific to this type manager.
 *  -fname string ...   String to display for an input pathname. This option
 *                      can be repeated up to the number of input pathnames
 *                      supplied, including the base_pname (which is
 *                      considered to be the first pathname.) The xmerge
 *                      program should use string[n] as a label for pname[n],
 *                      if string[n] was supplied.
 *  pname pname ...	List of pathnames of cleartext data for versions
 *			contributing to merge.
 *
 * Result:
 *  MGR_MERGE_OK	The merger was completed successfully.
 *  MGR_MERGE_ABORT	The merger was aborted due to conflicts.
 *  MGR_FAILED		Manager operation failed.
 */

/****************************************************************************
 * context_merge
 * Post-process the group of files that were merged or xmerged by this 
 * type manager in the current "merge session". This method is optional, 
 * and only a subset of ClearCase merge operations will invoke it. 
 * Currently this is limited to UCM deliver and rebase.
 *
 * If ClearCase intends to invoke this method following a merge or xmerge, 
 * the environment variable CCASE_WILL_CONTEXT_MERGE will be set during 
 * those methods.
 *
 * Command line:
 *  context_merge [-abort] pname
 *
 * Arguments:
 *
 *  -abort		Cancels the command instead of engaging in a user 
 *			interaction; a merge takes place only if it is 
 *			completely automatic.
 *  pname 		Pathname of a temporary file that itself contains 
 *			a list of pathnames, each of which was successfully 
 *			processed by the [x]merge method of this type manager 
 *			(or would have been had it not been a trivial merge), 
 *			and remains checkedout. One filename is listed per line. 
 *			Since each pathname will refer to a checkedout file 
 *			in the view's file area (in the MVFS for dynamic 
 *			views and in the copy area for snapshot views), 
 *			ClearCase commands such as properties or history may 
 *			be invoked using that pathname.
 *
 * Result:
 *  MGR_MERGE_OK	The merger was completed successfully.
 *  MGR_MERGE_ABORT	The merger was aborted due to conflicts.
 *  MGR_FAILED	Manager operation failed.
 */

/****************************************************************************
 * annotate
 * Fetch the annotated data for a version.
 * Extract the annotated data for the requested version from the indicated
 * source container into the supplied output pathname.
 * For more information, see cleartool annotate.
 *
 * Command line:
 *  annotate [-all] source_container_pname output_pname version_oid
 *
 * Default:		Fetch annotated data for requested version only.
 *			Indicate that all lines are INCLUDED.
 *
 * Arguments:
 *  -all		Fetch annotated data for all versions.  Indicate for
 *			each line whether it is still INCLUDED in the requested
 *			version, was included in an ancestor of the version but
 *			was DELETED, or is UNRELATED to the version.
 *  source_container_pname
 *			Pathname of source container for the version.
 *  output_pname	Pathname in which to store annotated data for version.
 *  version_oid		Object id of version.
 *			If not -all, fetch and annotate data for this version.
 *			If -all, fetch data for all versions of element and
 *			annotate them with respect to this version.
 *
 * Result:
 *  MGR_OK		Data was successfully fetched to output_pname.
 *  MGR_FAILED		Manager operation failed.
 *
 * Output file:
 * Each line of the output file has the format:
 *  code version-oid[,deleted-version-oid] line-of-data
 * where:
 *  code		indicates the relationship of this line to the version
 *			fetched (a mgr_annotate_code_t value)
 *  version-oid		object id of the version that first included the line
 *			of data
 *  deleted-version-oid	if line has been deleted, object id of the version that
 *			deleted the data
 *  line-of-data	line of data
 */

/****************************************************************************
 * get_cont_info
 * Fetch the information describing the contents of a container.
 * Extract the version, etc information from the source container
 * into the supplied output pathname.
 *
 * Command line:
 *  get_cont_info source_container_pname output_pname
 *
 * Arguments:
 *  source_container_pname
 *			Pathname of source container for the version.
 *  output_pname	Pathname in which to store container description
 *                      information.
 *
 * Result:
 *  MGR_OK		The manager maintains version identification information
 *                      and that data was successfully fetched to output_pname.
 *                      The output file must have at least one MGR_INFO_VERSION
 *                      record in it.
 *  MGR_FAILED		Manager operation failed.
 *  MGR_INFO_NO_INFO    The manager does not maintain version identify
 *                      information.  This response is only legal for
 *                      container-per-version style managers (also known 
 *                      as 'whole copy' style managers).
 *                      Such managers can still legally share a container
 *                      between a branch's predecessor version (its sprout 
 *                      point) and the the branch's branch-version-0 version.
 *                      (Note: a result of MGR_OK with an empty output file
 *                      is considered an error).
 *
 * Output file:
 * An unordered set of version, branch and element identification records,
 * with a single information record per line.
 * Only the version info records are mandatory. See the previous descriptions 
 * for the information record definitions:
 *    MGR_INFO_VERSION
 *    MGR_INFO_BRANCH
 *    MGR_INFO_ELEMENT
 */

#endif
